import 'package:snake/models/activity/cell_location.dart';
import 'package:snake/models/settings/settings_activity.dart';

typedef SnakeCells = List<CellLocation>;

enum SnakeDirection { top, left, bottom, right }

class Snake {
  // Snake's cells: last in list is head
  SnakeCells cells;
  SnakeDirection direction;

  Snake({
    required this.cells,
    required this.direction,
  });

  factory Snake.create(ActivitySettings activitySettings) {
    // Default init snake size: 3

    // ~ center cell:
    final int middleColumn = activitySettings.boardSize ~/ 2;
    final int middleRow = activitySettings.boardSize ~/ 2;

    SnakeCells cells = [
      CellLocation.go(middleRow, middleColumn),
      CellLocation.go(middleRow, middleColumn),
      CellLocation.go(middleRow, middleColumn),
    ];

    return Snake(
      cells: cells,
      direction: SnakeDirection.right,
    );
  }

  CellLocation get head => cells[cells.length - 1];

  @override
  String toString() {
    return '$Snake(${toJson()})';
  }

  Map<String, dynamic>? toJson() {
    return <String, dynamic>{
      'cells': cells,
      'direction': direction.toString(),
    };
  }
}
